@@ -35,7 +35,6 @@
 #include <geos/geom/prep/PreparedGeometry.h>
 #include <geos/geom/prep/PreparedGeometryFactory.h>
 #include <geos/geom/util/Densifier.h>
-#include <geos/operation/overlay/OverlayOp.h>
 #include <geos/operation/overlay/snap/GeometrySnapper.h>
 #include <geos/operation/overlayng/OverlayNG.h>
 #include <geos/operation/overlayng/OverlayNGRobust.h>
@@ -46,6 +45,8 @@
 #include <geos/operation/polygonize/BuildArea.h>
 #include <geos/operation/valid/MakeValid.h>
 #include <geos/precision/MinimumClearance.h>
+#include <geos/simplify/TopologyPreservingSimplifier.h>
+#include <geos/simplify/DouglasPeuckerSimplifier.h>
 #include <geos/util.h>
 #include <geos/util/Interrupt.h>
 #include <geos/io/WKBReader.h>
@@ -309,7 +310,7 @@ XMLTester::XMLTester()
     testValidOutput(false),
     testValidInput(false),
     sqlOutput(false),
-    HEXWKB_output(true)
+    HEXWKB_output(false)
 {
     setVerbosityLevel(0);
 }
@@ -435,7 +436,7 @@ XMLTester::run(const std::string& source)
     const tinyxml2::XMLNode* node = xml.FirstChildElement("run");
 
     if(! node) {
-        throw(runtime_error("Document has no childs"));
+        throw(runtime_error("Document has no children"));
     }
 
     parseRun(node);
@@ -704,6 +705,8 @@ XMLTester::parseCase(const tinyxml2::XMLNode* node)
         std::cout << *curr_file << ":";
         std::cout << " case" << caseCount << ":";
         std::cout << " skipped (" << thrownException << ")." << std::endl;
+        if (gA) delete gA;
+        if (gB) delete gB;
         return;
     }
 
@@ -1528,7 +1531,7 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
             }
         }
 
-        // equalsTopo() is synomym for equals() in JTS
+        // equalsTopo() is synonym for equals() in JTS
         else if(opName == "equalstopo") {
             geom::Geometry* g1 = opArg1 == "B" ? gB : gA;
             geom::Geometry* g2 = opArg2 == "B" ? gB : gA;
@@ -1866,12 +1869,7 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
 
 
             auto polys = plgnzr.getPolygons();
-            std::vector<geom::Geometry*>* newgeoms = new std::vector<geom::Geometry*>;
-            for(unsigned int i = 0; i < polys.size(); i++) {
-                newgeoms->push_back(polys[i].release());
-            }
-
-            GeomPtr gRealRes(factory->createGeometryCollection(newgeoms));
+            GeomPtr gRealRes(factory->createGeometryCollection(std::move(polys)));
             gRealRes->normalize();
 
 
@@ -2197,6 +2195,41 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
             }
         }
 
+        else if(opName == "simplifydp" || opName == "simplifytp")
+        {
+            geom::Geometry* p_gT = gA;
+            if((opArg1 == "B" || opArg1 == "b") && gB) {
+                p_gT = gB;
+            }
+
+            GeomPtr gRes(parseGeometry(opRes, "expected"));
+            gRes->normalize();
+
+            profile.start();
+
+            GeomPtr gRealRes;
+            double tolerance = std::atof(opArg2.c_str());
+
+            if (opName == "simplifydp") {
+                gRealRes = geos::simplify::DouglasPeuckerSimplifier::simplify(p_gT, tolerance);
+            }
+            else {
+                gRealRes = geos::simplify::TopologyPreservingSimplifier::simplify(p_gT, tolerance);
+            }
+
+            profile.stop();
+            gRealRes->normalize();
+
+            actual_result = printGeom(gRealRes.get());
+            expected_result = printGeom(gRes.get());
+
+            success = gRealRes.get()->equalsExact(gRes.get(), 0.000001) ? 1 : 0;
+
+            if(testValidOutput) {
+                success &= int(testValid(gRealRes.get(), "result"));
+            }
+        }
+
         else {
             std::cerr << *curr_file << ":";
             std::cerr << " case" << caseCount << ":";
@@ -2220,7 +2253,7 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
         }
     }
     catch(...) {
-        std::cerr << "Unknown EXEPTION on case "
+        std::cerr << "Unknown EXCEPTION on case "
                   << caseCount
                   << std::endl;
         actual_result = "Unknown exception thrown";
